# -*- coding:utf8 -*-
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/docUnderstanding/backend/dboperation.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/docUnderstanding/backend/dboperation.proto',
  package='docUnderstanding',
  syntax='proto3',
  serialized_options=_b('ZHgit.laiye.com/laiye-backend-repos/im-saas-protos-golang/docUnderstanding'),
  serialized_pb=_b('\n1protos/docUnderstanding/backend/dboperation.proto\x12\x10\x64ocUnderstanding\"!\n\x12\x44\x62OperationRequest\x12\x0b\n\x03sql\x18\x01 \x01(\t\"F\n\x0b\x44\x62\x44\x61taField\x12\x0b\n\x03key\x18\x01 \x01(\t\x12*\n\x04type\x18\x02 \x01(\x0e\x32\x1c.docUnderstanding.DbDataType\"#\n\x11\x44\x62OperationRecord\x12\x0e\n\x06values\x18\x01 \x03(\t\"z\n\x13\x44\x62OperationResponse\x12-\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x1d.docUnderstanding.DbDataField\x12\x34\n\x07records\x18\x02 \x03(\x0b\x32#.docUnderstanding.DbOperationRecord*%\n\nDbDataType\x12\x17\n\x13\x44\x42_DATA_TYPE_STRING\x10\x00\x32n\n\x12\x44\x62OperationService\x12X\n\x07Operate\x12$.docUnderstanding.DbOperationRequest\x1a%.docUnderstanding.DbOperationResponse\"\x00\x42JZHgit.laiye.com/laiye-backend-repos/im-saas-protos-golang/docUnderstandingb\x06proto3')
)

_DBDATATYPE = _descriptor.EnumDescriptor(
  name='DbDataType',
  full_name='docUnderstanding.DbDataType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DB_DATA_TYPE_STRING', index=0, number=0,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=339,
  serialized_end=376,
)
_sym_db.RegisterEnumDescriptor(_DBDATATYPE)

DbDataType = enum_type_wrapper.EnumTypeWrapper(_DBDATATYPE)
DB_DATA_TYPE_STRING = 0



_DBOPERATIONREQUEST = _descriptor.Descriptor(
  name='DbOperationRequest',
  full_name='docUnderstanding.DbOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sql', full_name='docUnderstanding.DbOperationRequest.sql', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=104,
)


_DBDATAFIELD = _descriptor.Descriptor(
  name='DbDataField',
  full_name='docUnderstanding.DbDataField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='docUnderstanding.DbDataField.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='docUnderstanding.DbDataField.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=176,
)


_DBOPERATIONRECORD = _descriptor.Descriptor(
  name='DbOperationRecord',
  full_name='docUnderstanding.DbOperationRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='docUnderstanding.DbOperationRecord.values', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=213,
)


_DBOPERATIONRESPONSE = _descriptor.Descriptor(
  name='DbOperationResponse',
  full_name='docUnderstanding.DbOperationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='docUnderstanding.DbOperationResponse.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='records', full_name='docUnderstanding.DbOperationResponse.records', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=337,
)

_DBDATAFIELD.fields_by_name['type'].enum_type = _DBDATATYPE
_DBOPERATIONRESPONSE.fields_by_name['fields'].message_type = _DBDATAFIELD
_DBOPERATIONRESPONSE.fields_by_name['records'].message_type = _DBOPERATIONRECORD
DESCRIPTOR.message_types_by_name['DbOperationRequest'] = _DBOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['DbDataField'] = _DBDATAFIELD
DESCRIPTOR.message_types_by_name['DbOperationRecord'] = _DBOPERATIONRECORD
DESCRIPTOR.message_types_by_name['DbOperationResponse'] = _DBOPERATIONRESPONSE
DESCRIPTOR.enum_types_by_name['DbDataType'] = _DBDATATYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DbOperationRequest = _reflection.GeneratedProtocolMessageType('DbOperationRequest', (_message.Message,), dict(
  DESCRIPTOR = _DBOPERATIONREQUEST,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbOperationRequest)
  ))
_sym_db.RegisterMessage(DbOperationRequest)

DbDataField = _reflection.GeneratedProtocolMessageType('DbDataField', (_message.Message,), dict(
  DESCRIPTOR = _DBDATAFIELD,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbDataField)
  ))
_sym_db.RegisterMessage(DbDataField)

DbOperationRecord = _reflection.GeneratedProtocolMessageType('DbOperationRecord', (_message.Message,), dict(
  DESCRIPTOR = _DBOPERATIONRECORD,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbOperationRecord)
  ))
_sym_db.RegisterMessage(DbOperationRecord)

DbOperationResponse = _reflection.GeneratedProtocolMessageType('DbOperationResponse', (_message.Message,), dict(
  DESCRIPTOR = _DBOPERATIONRESPONSE,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbOperationResponse)
  ))
_sym_db.RegisterMessage(DbOperationResponse)


DESCRIPTOR._options = None

_DBOPERATIONSERVICE = _descriptor.ServiceDescriptor(
  name='DbOperationService',
  full_name='docUnderstanding.DbOperationService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=378,
  serialized_end=488,
  methods=[
  _descriptor.MethodDescriptor(
    name='Operate',
    full_name='docUnderstanding.DbOperationService.Operate',
    index=0,
    containing_service=None,
    input_type=_DBOPERATIONREQUEST,
    output_type=_DBOPERATIONRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DBOPERATIONSERVICE)

DESCRIPTOR.services_by_name['DbOperationService'] = _DBOPERATIONSERVICE

# @@protoc_insertion_point(module_scope)